@@ -130,6 +130,7 @@ public class Wall3D extends Object3DBranch {
       wallAppearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
       wallAppearance.setMaterial(DEFAULT_MATERIAL);      
       wallAppearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);
+      wallAppearance.setCapability(Appearance.ALLOW_TEXTURE_READ);
       // Mix texture and wall color
       wallAppearance.setTextureAttributes(MODULATE_TEXTURE_ATTRIBUTES);
     }
@@ -692,7 +693,6 @@ public class Wall3D extends Object3DBranch {
   
   /**
    * Sets filled wall side appearance with its color, texture, transparency and visibility.
-   * @param f 
    */
   private void updateFilledWallSideAppearance(final Appearance wallSideAppearance, 
                                               final HomeTexture wallSideTexture,
@@ -739,7 +739,7 @@ public class Wall3D extends Object3DBranch {
     renderingAttributes.setVisible(drawingMode == HomeEnvironment.DrawingMode.OUTLINE 
         || drawingMode == HomeEnvironment.DrawingMode.FILL_AND_OUTLINE);
   }
-  
+
   /**
    * An area used to compute holes in walls. 
    */
