@@ -88,8 +88,44 @@ public class CatalogLight extends CatalogPieceOfFurniture implements Light {
                       LightSource [] lightSources,
                       float [][] modelRotation, String creator,
                       boolean resizable, BigDecimal price, BigDecimal valueAddedTaxPercentage) {
+    this(id, name, description, icon, planIcon, model, width, depth, height, elevation, movable,   
+        lightSources, modelRotation, creator, resizable, true, true, price, valueAddedTaxPercentage);
+  }
+         
+  /**
+   * Creates an unmodifiable catalog light of the default catalog.
+   * @param id    the id of the new light, or <code>null</code>
+   * @param name  the name of the new light
+   * @param description the description of the new light 
+   * @param icon content of the icon of the new light
+   * @param planIcon content of the icon of the new piece displayed in plan
+   * @param model content of the 3D model of the new light
+   * @param width  the width in centimeters of the new light
+   * @param depth  the depth in centimeters of the new light
+   * @param height  the height in centimeters of the new light
+   * @param elevation  the elevation in centimeters of the new light
+   * @param movable if <code>true</code>, the new light is movable
+   * @param lightSources the light sources of the new light
+   * @param modelRotation the rotation 3 by 3 matrix applied to the light model
+   * @param creator the creator of the model
+   * @param resizable if <code>true</code>, the size of the new light may be edited
+   * @param deformable if <code>true</code>, the width, depth and height of the new piece may 
+   *            change independently from each other
+   * @param texturable if <code>false</code> this piece should always keep the same color or texture.
+   * @param price the price of the new light, or <code>null</code> 
+   * @param valueAddedTaxPercentage the Value Added Tax percentage applied to the 
+   *             price of the new light or <code>null</code> 
+   * @since 3.0            
+   */
+  public CatalogLight(String id, String name, String description, 
+                      Content icon, Content planIcon, Content model, 
+                      float width, float depth, float height, float elevation, boolean movable, 
+                      LightSource [] lightSources,
+                      float [][] modelRotation, String creator,
+                      boolean resizable, boolean deformable, boolean texturable,
+                      BigDecimal price, BigDecimal valueAddedTaxPercentage) {
     super(id, name, description, icon, planIcon, model, width, depth, height, elevation, movable,   
-        modelRotation, creator, resizable, price, valueAddedTaxPercentage);
+        modelRotation, creator, resizable, deformable, texturable, price, valueAddedTaxPercentage);
     this.lightSources = lightSources;
   }
          
