@@ -64,6 +64,7 @@ public class Wall implements Serializable, Selectable {
   private Integer     rightSideColor;
   private HomeTexture rightSideTexture;
   private float       rightSideShininess;
+  private boolean     symmetric = true;
   
   private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
   private transient float [][] pointsCache;
@@ -613,7 +614,17 @@ public class Wall implements Serializable, Selectable {
         int wallAtStartRightSideStartPointIndex = wallAtStartPoints.length - 1;
         int wallAtStartLeftSideEndPointIndex = wallAtStartPoints.length / 2 - 1;
         int wallAtStartRightSideEndPointIndex = wallAtStartPoints.length / 2;
-        if (this.wallAtStart.getWallAtEnd() == this) {
+        boolean wallAtStartJoinedAtEnd = this.wallAtStart.getWallAtEnd() == this
+            // Check the coordinates when walls are joined to each other at both ends 
+            && (this.wallAtStart.getWallAtStart() != this
+                || (this.wallAtStart.xEnd == this.xStart
+                    && this.wallAtStart.yEnd == this.yStart));
+        boolean wallAtStartJoinedAtStart = this.wallAtStart.getWallAtStart() == this
+            // Check the coordinates when walls are joined to each other at both ends 
+            && (this.wallAtStart.getWallAtEnd() != this
+                || (this.wallAtStart.xStart == this.xStart
+                    && this.wallAtStart.yStart == this.yStart));
+        if (wallAtStartJoinedAtEnd) {
           computeIntersection(wallPoints [leftSideStartPointIndex], wallPoints [leftSideStartPointIndex + 1], 
               wallAtStartPoints [wallAtStartLeftSideEndPointIndex], wallAtStartPoints [wallAtStartLeftSideEndPointIndex - 1], limit);
           computeIntersection(wallPoints [rightSideStartPointIndex], wallPoints [rightSideStartPointIndex - 1],  
@@ -631,7 +642,7 @@ public class Wall implements Serializable, Selectable {
               wallPoints [rightSideStartPointIndex] = this.wallAtStart.pointsCache [wallAtStartRightSideEndPointIndex];
             }
           }
-        } else if (this.wallAtStart.getWallAtStart() == this) {
+        } else if (wallAtStartJoinedAtStart) {
           computeIntersection(wallPoints [leftSideStartPointIndex], wallPoints [leftSideStartPointIndex + 1], 
               wallAtStartPoints [wallAtStartRightSideStartPointIndex], wallAtStartPoints [wallAtStartRightSideStartPointIndex - 1], limit);
           computeIntersection(wallPoints [rightSideStartPointIndex], wallPoints [rightSideStartPointIndex - 1],  
@@ -661,7 +672,17 @@ public class Wall implements Serializable, Selectable {
         int wallAtEndRightSideStartPointIndex = wallAtEndPoints.length - 1;
         int wallAtEndLeftSideEndPointIndex = wallAtEndPoints.length / 2 - 1;
         int wallAtEndRightSideEndPointIndex = wallAtEndPoints.length / 2;
-        if (this.wallAtEnd.getWallAtStart() == this) {
+        boolean wallAtEndJoinedAtStart = this.wallAtEnd.getWallAtStart() == this
+            // Check the coordinates when walls are joined to each other at both ends 
+            && (this.wallAtEnd.getWallAtEnd() != this
+                || (this.wallAtEnd.xStart == this.xEnd
+                    && this.wallAtEnd.yStart == this.yEnd));
+        boolean wallAtEndJoinedAtEnd = this.wallAtEnd.getWallAtEnd() == this
+            // Check the coordinates when walls are joined to each other at both ends 
+            && (this.wallAtEnd.getWallAtStart() != this
+                || (this.wallAtEnd.xEnd == this.xEnd
+                    && this.wallAtEnd.yEnd == this.yEnd));
+        if (wallAtEndJoinedAtStart) {
           computeIntersection(wallPoints [leftSideEndPointIndex], wallPoints [leftSideEndPointIndex - 1], 
               wallAtEndPoints [wallAtEndLeftSideStartPointIndex], wallAtEndPoints [wallAtEndLeftSideStartPointIndex + 1], limit);
           computeIntersection(wallPoints [rightSideEndPointIndex], wallPoints [rightSideEndPointIndex + 1], 
@@ -679,7 +700,7 @@ public class Wall implements Serializable, Selectable {
               wallPoints [rightSideEndPointIndex] = this.wallAtEnd.pointsCache [wallAtEndRightSideStartPointIndex];
             }
           }
-        } else if (this.wallAtEnd.getWallAtEnd() == this) {
+        } else if (wallAtEndJoinedAtEnd) {
           computeIntersection(wallPoints [leftSideEndPointIndex], wallPoints [leftSideEndPointIndex - 1],  
               wallAtEndPoints [wallAtEndRightSideEndPointIndex], wallAtEndPoints [wallAtEndRightSideEndPointIndex + 1], limit);
           computeIntersection(wallPoints [rightSideEndPointIndex], wallPoints [rightSideEndPointIndex + 1], 
@@ -710,7 +731,7 @@ public class Wall implements Serializable, Selectable {
   }
 
   /**
-   * Compute the rectangle or the circle arc of a wall with its thickness.
+   * Computes the rectangle or the circle arc of a wall with its thickness.
    */  
   private float [][] getUnjoinedShapePoints() {
     if (this.arcExtent != null
@@ -724,31 +745,26 @@ public class Wall implements Serializable, Selectable {
       float interiorArcRadius = Math.max(0, arcCircleRadius - this.thickness / 2);
       float exteriorArcLength = exteriorArcRadius * Math.abs(this.arcExtent);
       float angleDelta = this.arcExtent / (float)Math.sqrt(exteriorArcLength);
-      List<float[]> wallPoints = new ArrayList<float[]>(100);
-      int i = 0;
-      for (float angle = this.arcExtent; angleDelta > 0 ? angle >= angleDelta * 0.1f : angle <= -angleDelta * 0.1f; angle -= angleDelta, i++) {
-        double cos = Math.cos(startAngle + angle);
-        double sin = Math.sin(startAngle + angle);
-        float [] interiorArcPoint = new float [] {(float)(arcCircleCenter [0] + interiorArcRadius * cos), 
-                                                  (float)(arcCircleCenter [1] - interiorArcRadius * sin)};
-        float [] exteriorArcPoint = new float [] {(float)(arcCircleCenter [0] + exteriorArcRadius * cos), 
-                                                  (float)(arcCircleCenter [1] - exteriorArcRadius * sin)};
-        if (angleDelta > 0) {
-          wallPoints.add(i, interiorArcPoint);
-          wallPoints.add(wallPoints.size() - 1 - i, exteriorArcPoint);
-        } else {
-          wallPoints.add(i, exteriorArcPoint);
-          wallPoints.add(wallPoints.size() - 1 - i, interiorArcPoint);
+      int angleStepCount = (int)(this.arcExtent / angleDelta);
+      List<float[]> wallPoints = new ArrayList<float[]>((angleStepCount + 2) * 2);      
+      if (this.symmetric) {
+        if (Math.abs(this.arcExtent - angleStepCount * angleDelta) > 1E-6) {
+          angleDelta = this.arcExtent / ++angleStepCount;
         }
+        for (int i = 0; i <= angleStepCount; i++) {
+          computeRoundWallShapePoint(wallPoints, startAngle + this.arcExtent - i * angleDelta, i, angleDelta, 
+              arcCircleCenter, exteriorArcRadius, interiorArcRadius);
+        }
+      } else {
+        // Don't change the way walls were computed in version 3.0 to ensure they exactly look the same
+        int i = 0;
+        for (float angle = this.arcExtent; angleDelta > 0 ? angle >= angleDelta * 0.1f : angle <= -angleDelta * 0.1f; angle -= angleDelta, i++) {
+          computeRoundWallShapePoint(wallPoints, startAngle + angle, i, angleDelta, 
+              arcCircleCenter, exteriorArcRadius, interiorArcRadius);
+        }
+        computeRoundWallShapePoint(wallPoints, startAngle, i, angleDelta, 
+            arcCircleCenter, exteriorArcRadius, interiorArcRadius);
       }
-      double cos = Math.cos(startAngle);
-      double sin = Math.sin(startAngle);
-      float [] interiorStartPoint = new float [] {(float)(arcCircleCenter [0] + interiorArcRadius * cos), 
-                                                  (float)(arcCircleCenter [1] - interiorArcRadius * sin)};
-      float [] exteriorStartPoint = new float [] {(float)(arcCircleCenter [0] + exteriorArcRadius * cos), 
-                                                  (float)(arcCircleCenter [1] - exteriorArcRadius * sin)};
-      wallPoints.add(i, angleDelta > 0  ? interiorStartPoint  : exteriorStartPoint);
-      wallPoints.add(wallPoints.size() - 1 - i, angleDelta > 0  ? exteriorStartPoint  : interiorStartPoint);
       return wallPoints.toArray(new float [wallPoints.size()][]);
     } else { 
       double angle = Math.atan2(this.yEnd - this.yStart, 
@@ -762,6 +778,26 @@ public class Wall implements Serializable, Selectable {
           {this.xStart - dx, this.yStart + dy}};
     }
   }
+
+  /**
+   * Computes the exterior and interior arc points of a round wall at the given <code>index</code>.
+   */  
+  private void computeRoundWallShapePoint(List<float []> wallPoints, float angle, int index, float angleDelta, 
+                                          float [] arcCircleCenter, float exteriorArcRadius, float interiorArcRadius) {
+    double cos = Math.cos(angle);
+    double sin = Math.sin(angle);
+    float [] interiorArcPoint = new float [] {(float)(arcCircleCenter [0] + interiorArcRadius * cos), 
+                                              (float)(arcCircleCenter [1] - interiorArcRadius * sin)};
+    float [] exteriorArcPoint = new float [] {(float)(arcCircleCenter [0] + exteriorArcRadius * cos), 
+                                              (float)(arcCircleCenter [1] - exteriorArcRadius * sin)};
+    if (angleDelta > 0) {
+      wallPoints.add(index, interiorArcPoint);
+      wallPoints.add(wallPoints.size() - 1 - index, exteriorArcPoint);
+    } else {
+      wallPoints.add(index, exteriorArcPoint);
+      wallPoints.add(wallPoints.size() - 1 - index, interiorArcPoint);
+    }
+  }
   
   /**
    * Compute the intersection between the line that joins <code>point1</code> to <code>point2</code>
