@@ -258,7 +258,11 @@ public class CompassPanel extends JPanel implements DialogView {
                                                       boolean cellHasFocus) {
           String timeZoneId = (String)value;
           String timeZoneDisplayName = TimeZone.getTimeZone(timeZoneId).getDisplayName();
-          value = timeZoneId + " - " + timeZoneDisplayName;
+          if (OperatingSystem.isMacOSX()) {
+            value = timeZoneId + " - " + timeZoneDisplayName;
+          } else {
+            setToolTipText(timeZoneId + " - " + timeZoneDisplayName);
+          }
           return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
         }
       });
@@ -270,6 +274,7 @@ public class CompassPanel extends JPanel implements DialogView {
           controller.addPropertyChangeListener(CompassController.Property.TIME_ZONE, timeZoneChangeListener);
         }
       });
+    this.timeZoneComboBox.setPrototypeDisplayValue("GMT");
     
     this.northDirectionLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences, CompassPanel.class, "northDirectionLabel.text"));
     // Create a spinner model able to choose an angle modulo 360
@@ -405,7 +410,7 @@ public class CompassPanel extends JPanel implements DialogView {
         GridBagConstraints.NONE, labelInsets, 0, 0));
     compassRosePanel.add(this.xSpinner, new GridBagConstraints(
         1, 0, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, componentInsets, -10, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 20), -10, 0));
     compassRosePanel.add(this.visibleCheckBox, new GridBagConstraints(
         2, 0, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.HORIZONTAL, lastComponentInsets, 0, 0));
@@ -414,7 +419,7 @@ public class CompassPanel extends JPanel implements DialogView {
         GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
     compassRosePanel.add(this.ySpinner, new GridBagConstraints(
         1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 10), -10, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 20), -10, 0));
     compassRosePanel.add(this.diameterLabel, new GridBagConstraints(
         2, 1, 1, 1, 0, 0, labelAlignment, 
         GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
@@ -453,8 +458,8 @@ public class CompassPanel extends JPanel implements DialogView {
         GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
     geographicLocationPanel.add(this.timeZoneComboBox, new GridBagConstraints(
         3, 1, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 40, 0));
-    this.timeZoneComboBox.setPreferredSize(new Dimension(this.latitudeSpinner.getPreferredSize().width, 
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
+    this.timeZoneComboBox.setPreferredSize(new Dimension(this.latitudeSpinner.getPreferredSize().width + 60, 
         this.timeZoneComboBox.getPreferredSize().height));
     add(geographicLocationPanel, new GridBagConstraints(
         0, 1, 1, 1, 1, 0, GridBagConstraints.CENTER, 
