@@ -39,8 +39,10 @@ import java.awt.event.ItemListener;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseMotionAdapter;
+import java.awt.image.BufferedImage;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
+import java.io.IOException;
 import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -48,6 +50,7 @@ import java.util.Collections;
 import java.util.List;
 import java.util.regex.Pattern;
 
+import javax.imageio.ImageIO;
 import javax.swing.AbstractAction;
 import javax.swing.AbstractListModel;
 import javax.swing.DefaultComboBoxModel;
@@ -131,17 +134,29 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
           int index = locationToIndex(ev.getPoint());
           if (index != -1) {
             CatalogPieceOfFurniture piece = (CatalogPieceOfFurniture)getModel().getElementAt(index);
-            String tooltip = "<html><center>&nbsp;- <b>" + piece.getCategory().getName() + "</b> -&nbsp;"
-                + "<br><center>&nbsp;<b>" + piece.getName() + "</b>&nbsp;";
+            String tooltip = "<html><table><tr><td align='center'>- <b>" + piece.getCategory().getName() + "</b> -" 
+                + "<br><b>" + piece.getName() + "</b>";
             if (piece.getCreator() != null) {
               tooltip += "<br>" + preferences.getLocalizedString(FurnitureCatalogTree.class, 
-                  "tooltipCreator", piece.getCreator());
+                  "tooltipCreator", piece.getCreator() + "</td></tr>");
             }
             if (piece.getIcon() instanceof URLContent) {
-              tooltip += "<br><img width='128' height='128' src='" 
-                + ((URLContent)piece.getIcon()).getURL() + "'>"; 
+              try {
+                // Ensure image will always be viewed in a 128x128 pixels cell
+                BufferedImage image = ImageIO.read(((URLContent)piece.getIcon()).getURL());
+                if (image == null) {
+                  return null;
+                }
+                int width = Math.round(128f * Math.min(1, image.getWidth() / image.getHeight()));
+                int height = Math.round((float)width * image.getHeight() / image.getWidth());
+                tooltip += "<tr><td width='128' height='128' align='center' valign='middle'><img width='" + width 
+                    + "' height='" + height + "' src='" 
+                    + ((URLContent)piece.getIcon()).getURL() + "'></td></tr>";
+              } catch (IOException ex) {
+                return null;
+              }
             }
-            return tooltip;
+            return tooltip + "</table>";
           } else {
             return null;
           }
@@ -274,22 +289,24 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
       this.searchTextField.putClientProperty("JTextField.variant", "search");
     } 
     
-    preferences.addPropertyChangeListener(UserPreferences.Property.LANGUAGE, new LanguageChangeListener(this));
+    PreferencesChangeListener preferencesChangeListener = new PreferencesChangeListener(this);
+    preferences.addPropertyChangeListener(UserPreferences.Property.LANGUAGE, preferencesChangeListener);
+    catalog.addFurnitureListener(preferencesChangeListener);
   }
   
   /**
-   * Preferences property listener bound to this component with a weak reference to avoid
+   * Language and catalog listener bound to this component with a weak reference to avoid
    * strong link between preferences and this component.  
    */
-  public static class LanguageChangeListener implements PropertyChangeListener {
+  public static class PreferencesChangeListener implements PropertyChangeListener, CollectionListener<CatalogPieceOfFurniture> {
     private final WeakReference<FurnitureCatalogListPanel> furnitureCatalogPanel;
 
-    public LanguageChangeListener(FurnitureCatalogListPanel furnitureCatalogPanel) {
+    public PreferencesChangeListener(FurnitureCatalogListPanel furnitureCatalogPanel) {
       this.furnitureCatalogPanel = new WeakReference<FurnitureCatalogListPanel>(furnitureCatalogPanel);
     }
 
     public void propertyChange(PropertyChangeEvent ev) {
-      // If furniture table was garbage collected, remove this listener from preferences
+      // If panel was garbage collected, remove this listener from preferences
       FurnitureCatalogListPanel furnitureCatalogPanel = this.furnitureCatalogPanel.get();
       UserPreferences preferences = (UserPreferences)ev.getSource();
       if (furnitureCatalogPanel == null) {
@@ -307,6 +324,26 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
         furnitureCatalogPanel.categoryFilterComboBox.setModel(new DefaultComboBoxModel(categories.toArray()));    
       }
     }
+    
+    public void collectionChanged(CollectionEvent<CatalogPieceOfFurniture> ev) {
+      // If panel was garbage collected, remove this listener from catalog
+      FurnitureCatalogListPanel furnitureCatalogPanel = this.furnitureCatalogPanel.get();
+      FurnitureCatalog catalog = (FurnitureCatalog)ev.getSource();
+      if (furnitureCatalogPanel == null) {
+        catalog.removeFurnitureListener(this);
+      } else {
+        DefaultComboBoxModel model = 
+            (DefaultComboBoxModel)furnitureCatalogPanel.categoryFilterComboBox.getModel();
+        FurnitureCategory category = ev.getItem().getCategory();
+        List<FurnitureCategory> categories = catalog.getCategories();
+        if (!categories.contains(category)) {
+          model.removeElement(category);
+          furnitureCatalogPanel.categoryFilterComboBox.setSelectedIndex(0);
+        } else if (model.getIndexOf(category) == -1) {
+          model.insertElementAt(category, categories.indexOf(category) + 1);
+        }
+      }
+    }
   }
 
   /**
@@ -596,7 +633,7 @@ public class FurnitureCatalogListPanel extends JPanel implements View {
       for (FurnitureCategory category : this.catalog.getCategories()) {
         for (CatalogPieceOfFurniture piece : category.getFurniture()) {
           if ((this.filterCategory == null
-               || piece.getCategory() == this.filterCategory)
+               || piece.getCategory().equals(this.filterCategory))
                && (this.filterNamePattern.matcher(piece.getName()).matches()
                    || this.filterNamePattern.matcher(piece.getCategory().getName()).matches()                   
                    || (piece.getCreator() != null && this.filterNamePattern.matcher(piece.getCreator()).matches())
