@@ -20,12 +20,15 @@
 package com.eteks.sweethome3d.swing;
 
 import java.awt.BorderLayout;
+import java.awt.Color;
 import java.awt.Component;
 import java.awt.ComponentOrientation;
 import java.awt.Container;
 import java.awt.Cursor;
 import java.awt.DefaultFocusTraversalPolicy;
 import java.awt.Dimension;
+import java.awt.GridBagConstraints;
+import java.awt.GridBagLayout;
 import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.beans.PropertyChangeEvent;
@@ -40,7 +43,6 @@ import javax.jnlp.ServiceManager;
 import javax.jnlp.UnavailableServiceException;
 import javax.swing.Action;
 import javax.swing.ActionMap;
-import javax.swing.Box;
 import javax.swing.ImageIcon;
 import javax.swing.InputMap;
 import javax.swing.JButton;
@@ -57,6 +59,10 @@ import javax.swing.event.DocumentEvent;
 import javax.swing.event.DocumentListener;
 import javax.swing.event.HyperlinkEvent;
 import javax.swing.event.HyperlinkListener;
+import javax.swing.text.BadLocationException;
+import javax.swing.text.DefaultHighlighter;
+import javax.swing.text.Element;
+import javax.swing.text.Highlighter;
 
 import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.tools.OperatingSystem;
@@ -80,7 +86,7 @@ public class HelpPane extends JRootPane implements HelpView {
                   final HelpController controller) {
     this.preferences = preferences;
     createActions(preferences, controller);
-    createComponents(preferences);
+    createComponents(preferences, controller);
     setMnemonics(preferences);
     layoutComponents();
     addLanguageListener(preferences);
@@ -94,6 +100,7 @@ public class HelpPane extends JRootPane implements HelpView {
         new PropertyChangeListener() {
           public void propertyChange(PropertyChangeEvent ev) {
             setPage(controller.getHelpPage());
+            highlightText(controller.getHighlightedText());
           }
         });
     controller.addPropertyChangeListener(HelpController.Property.BROWSER_PAGE, 
@@ -204,7 +211,7 @@ public class HelpPane extends JRootPane implements HelpView {
   /**
    * Creates the components displayed by this view.
   */
-  private void createComponents(UserPreferences preferences) {
+  private void createComponents(UserPreferences preferences, final HelpController controller) {
     this.searchLabel = new JLabel(SwingTools.getLocalizedLabelText(preferences, HelpPane.class, "searchLabel.text"));
     this.searchTextField = new JTextField(12);
     // Under Mac OS 10.5 use client properties to use search text field look and feel
@@ -218,6 +225,7 @@ public class HelpPane extends JRootPane implements HelpView {
     this.searchTextField.getDocument().addDocumentListener(new DocumentListener() {
         public void changedUpdate(DocumentEvent ev) {
           getActionMap().get(ActionType.SEARCH).setEnabled(searchTextField.getText().trim().length() > 0);
+          controller.setHighlightedText(searchTextField.getText());
         }
     
         public void insertUpdate(DocumentEvent ev) {
@@ -234,6 +242,14 @@ public class HelpPane extends JRootPane implements HelpView {
     this.helpEditorPane.setEditable(false);
     this.helpEditorPane.setContentType("text/html");
     this.helpEditorPane.putClientProperty(JEditorPane.W3C_LENGTH_UNITS, Boolean.TRUE);
+    this.helpEditorPane.setHighlighter(new DefaultHighlighter());
+    PropertyChangeListener highlightingTextListener = new PropertyChangeListener() {
+        public void propertyChange(PropertyChangeEvent ev) {
+          highlightText(controller.getHighlightedText());
+        }
+      };
+    controller.addPropertyChangeListener(HelpController.Property.HIGHLIGHTED_TEXT, highlightingTextListener);
+    this.helpEditorPane.addPropertyChangeListener("page", highlightingTextListener);
     
     setFocusTraversalPolicy(new DefaultFocusTraversalPolicy() {
         @Override
@@ -243,6 +259,47 @@ public class HelpPane extends JRootPane implements HelpView {
       });
   }
 
+  /**
+   * Highlights the words of the given text in HTML pane.
+   */
+  private void highlightText(String highlightedText) {
+    DefaultHighlighter.DefaultHighlightPainter highlightPainter = 
+        new DefaultHighlighter.DefaultHighlightPainter(new Color(255, 204, 51));
+    this.helpEditorPane.getHighlighter().removeAllHighlights();
+    if (highlightedText != null) {
+      String [] highlightedWords = highlightedText.split("\\s");
+      for (int i = 0; i < highlightedWords.length; i++) {
+        highlightedWords [i] = highlightedWords [i].toLowerCase().trim();
+      }                            
+      highlightWords(this.helpEditorPane.getDocument().getDefaultRootElement(), 
+          highlightedWords, highlightPainter);
+    }
+  }
+
+  private void highlightWords(Element element, 
+                              String [] highlightedWords, 
+                              Highlighter.HighlightPainter highlightPainter) {
+    if (element.isLeaf()) {
+      int startOffset = element.getStartOffset();
+      try {
+        String text = element.getDocument().getText(element.getStartOffset(), 
+            element.getEndOffset() - startOffset).toLowerCase();
+        for (String highlightedWord : highlightedWords) {
+          for (int index = 0; index < text.length() - 1 &&  (index = text.indexOf(highlightedWord, index)) >= 0; index += highlightedWord.length() + 1) {
+            this.helpEditorPane.getHighlighter().addHighlight(
+                startOffset + index, startOffset + index + highlightedWord.length(), highlightPainter);
+          }
+        }
+      } catch (BadLocationException ex) {
+        // Ignore unexpected exception
+      }
+    } else {
+      for (int i = 0, n = element.getElementCount(); i < n; i++) {
+        highlightWords(element.getElement(i), highlightedWords, highlightPainter);
+      }
+    }
+  }
+
   /**
    * Sets components mnemonics and label / component associations.
    */
@@ -260,9 +317,15 @@ public class HelpPane extends JRootPane implements HelpView {
   private void layoutComponents() {
     final JToolBar toolBar = new JToolBar();
     toolBar.setFloatable(false);
+    // Change layout because BoxLayout glue doesn't work well under Linux
+    toolBar.setLayout(new GridBagLayout());
     ActionMap actions = getActionMap();    
-    toolBar.add(actions.get(ActionType.SHOW_PREVIOUS));
-    toolBar.add(actions.get(ActionType.SHOW_NEXT));
+    toolBar.add(new JButton(actions.get(ActionType.SHOW_PREVIOUS)),
+        new GridBagConstraints(0, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, 
+            GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
+    toolBar.add(new JButton(actions.get(ActionType.SHOW_NEXT)),
+        new GridBagConstraints(1, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, 
+            GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
     updateToolBarButtonsStyle(toolBar);
     toolBar.addPropertyChangeListener("componentOrientation", 
         new PropertyChangeListener () {
@@ -270,23 +333,28 @@ public class HelpPane extends JRootPane implements HelpView {
             updateToolBarButtonsStyle(toolBar);
           }
         });
-    toolBar.add(Box.createHorizontalStrut(5));
+    toolBar.add(new JLabel(),
+        new GridBagConstraints(2, 0, 1, 1, 1, 0, GridBagConstraints.CENTER, 
+            GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
     
-    toolBar.add(Box.createGlue());
     if (!OperatingSystem.isMacOSXLeopardOrSuperior()) {
-      toolBar.add(this.searchLabel);
-      toolBar.add(Box.createHorizontalStrut(2));
+      toolBar.add(this.searchLabel,
+          new GridBagConstraints(3, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, 
+              GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
     }
-    toolBar.add(this.searchTextField);
+    toolBar.add(this.searchTextField,
+        new GridBagConstraints(4, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, 
+            GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
     this.searchTextField.setMaximumSize(this.searchTextField.getPreferredSize());
     // Ignore search button under Mac OS X 10.5 (it's included in the search field)
     if (!OperatingSystem.isMacOSXLeopardOrSuperior()) {
-      toolBar.add(Box.createHorizontalStrut(2));
-      toolBar.add(actions.get(ActionType.SEARCH));
+      toolBar.add(new JButton(actions.get(ActionType.SEARCH)),
+          new GridBagConstraints(5, 0, 1, 1, 0, 0, GridBagConstraints.CENTER, 
+              GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));          
     }
     // Remove focusable property on buttons
     for (int i = 0, n = toolBar.getComponentCount(); i < n; i++) {      
-      Component component = toolBar.getComponentAtIndex(i);
+      Component component = toolBar.getComponent(i);
       if (component instanceof JButton) {
         component.setFocusable(false);
       }
@@ -305,13 +373,13 @@ public class HelpPane extends JRootPane implements HelpView {
     if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
       // Retrieve component orientation because Mac OS X 10.5 miserably doesn't it take into account 
       ComponentOrientation orientation = toolBar.getComponentOrientation();
-      JComponent previousButton = (JComponent)toolBar.getComponentAtIndex(0);
+      JComponent previousButton = (JComponent)toolBar.getComponent(0);
       previousButton.putClientProperty("JButton.buttonType", "segmentedTextured");
       previousButton.putClientProperty("JButton.segmentPosition", 
           orientation == ComponentOrientation.LEFT_TO_RIGHT 
             ? "first"
             : "last");
-      JComponent nextButton = (JComponent)toolBar.getComponentAtIndex(1);
+      JComponent nextButton = (JComponent)toolBar.getComponent(1);
       nextButton.putClientProperty("JButton.buttonType", "segmentedTextured");
       nextButton.putClientProperty("JButton.segmentPosition", 
           orientation == ComponentOrientation.LEFT_TO_RIGHT 
