@@ -62,6 +62,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -85,6 +86,7 @@ import javax.media.j3d.Group;
 import javax.media.j3d.IllegalRenderingStateException;
 import javax.media.j3d.J3DGraphics2D;
 import javax.media.j3d.Light;
+import javax.media.j3d.Link;
 import javax.media.j3d.Node;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.Texture;
@@ -95,7 +97,6 @@ import javax.media.j3d.TransparencyAttributes;
 import javax.media.j3d.View;
 import javax.swing.AbstractAction;
 import javax.swing.ActionMap;
-import javax.swing.Icon;
 import javax.swing.ImageIcon;
 import javax.swing.InputMap;
 import javax.swing.JButton;
@@ -106,6 +107,7 @@ import javax.swing.KeyStroke;
 import javax.swing.RepaintManager;
 import javax.swing.SwingUtilities;
 import javax.swing.Timer;
+import javax.swing.border.Border;
 import javax.swing.event.AncestorEvent;
 import javax.swing.event.AncestorListener;
 import javax.swing.event.ChangeEvent;
@@ -119,12 +121,10 @@ import javax.vecmath.Vector3f;
 
 import com.eteks.sweethome3d.j3d.Component3DManager;
 import com.eteks.sweethome3d.j3d.Ground3D;
-import com.eteks.sweethome3d.j3d.HomePieceOfFurniture3D;
 import com.eteks.sweethome3d.j3d.ModelManager;
 import com.eteks.sweethome3d.j3d.Object3DBranch;
-import com.eteks.sweethome3d.j3d.Room3D;
+import com.eteks.sweethome3d.j3d.Object3DBranchFactory;
 import com.eteks.sweethome3d.j3d.TextureManager;
-import com.eteks.sweethome3d.j3d.Wall3D;
 import com.eteks.sweethome3d.model.Camera;
 import com.eteks.sweethome3d.model.CollectionEvent;
 import com.eteks.sweethome3d.model.CollectionListener;
@@ -139,6 +139,7 @@ import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.model.Wall;
 import com.eteks.sweethome3d.tools.OperatingSystem;
 import com.eteks.sweethome3d.viewcontroller.HomeController3D;
+import com.eteks.sweethome3d.viewcontroller.Object3DFactory;
 import com.sun.j3d.utils.geometry.GeometryInfo;
 import com.sun.j3d.utils.universe.SimpleUniverse;
 import com.sun.j3d.utils.universe.Viewer;
@@ -156,10 +157,11 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   
   private final Home                               home;
   private final boolean                            displayShadowOnFloor;
+  private final Object3DFactory                    object3dFactory;
   private final Map<Selectable, Object3DBranch>    homeObjects = new HashMap<Selectable, Object3DBranch>();
   private Collection<Selectable>                   homeObjectsToUpdate;
   private Canvas3D                                 canvas3D;
-  private SimpleUniverse                           universe;
+  private SimpleUniverse                           onscreenUniverse;
   private Camera                                   camera;
   // Listeners bound to home that updates 3D scene objects
   private PropertyChangeListener                   cameraChangeListener;
@@ -179,6 +181,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   // Creating an offscreen buffer is a quite lengthy operation so we keep the last printed image in this field
   // This image should be set to null each time the 3D view changes
   private BufferedImage                            printedImage;
+  private SimpleUniverse                           offscreenUniverse;
   
   private JComponent                               navigationPanel;
   private ComponentListener                        navigationPanelListener;
@@ -212,7 +215,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   public HomeComponent3D(Home home, 
                          UserPreferences  preferences, 
                          boolean displayShadowOnFloor) {
-    this(home, preferences, displayShadowOnFloor, null);  
+    this(home, preferences, new Object3DBranchFactory(), displayShadowOnFloor, null);  
   }
   
   /**
@@ -223,7 +226,25 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   public HomeComponent3D(Home home,
                          UserPreferences  preferences,
                          HomeController3D controller) {
-    this(home, preferences, false, controller);    
+    this(home, preferences, new Object3DBranchFactory(), false, controller);    
+  }
+
+  /**
+   * Creates a 3D component that displays <code>home</code> walls, rooms and furniture.
+   * @param home the home to display in this component
+   * @param preferences user preferences
+   * @param object3DFactory a factory able to create 3D objects from <code>home</code> items.
+   *            The {@link Object3DFactory#createObject3D(Home, Selectable, boolean) createObject3D} of 
+   *            this factory is expected to return an instance of {@link Object3DBranch} in current implementation.
+   * @param controller the controller that manages modifications in <code>home</code>.           
+   * @throws IllegalRenderingStateException  if the canvas 3D displayed 
+   *            by this component couldn't be created.
+   */
+  public HomeComponent3D(Home home,
+                         UserPreferences  preferences,
+                         Object3DFactory  object3DFactory,
+                         HomeController3D controller) {
+    this(home, preferences, object3DFactory, false, controller);    
   }
 
   /**
@@ -233,10 +254,12 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
    */
   private HomeComponent3D(Home home,
                           UserPreferences  preferences,
+                          Object3DFactory  object3DFactory,
                           boolean displayShadowOnFloor,
                           HomeController3D controller) {
     this.home = home;
     this.displayShadowOnFloor = displayShadowOnFloor;
+    this.object3dFactory = object3DFactory;
 
     this.canvas3D = Component3DManager.getInstance().getOnscreenCanvas3D(new Component3DManager.RenderingObserver() {        
         public void canvas3DSwapped(Canvas3D canvas3D) {
@@ -294,7 +317,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
         // No support for navigation panel under Mac OS X Tiger 
         // (too unstable, may crash system at 3D view resizing)
         this.navigationPanel = createNavigationPanel(home, preferences, controller);
-        setNavigationPanelVisible(preferences.isNavigationPanelVisible());
+        setNavigationPanelVisible(preferences.isNavigationPanelVisible() && isVisible());
         preferences.addPropertyChangeListener(UserPreferences.Property.NAVIGATION_PANEL_VISIBLE, 
             new NavigationPanelChangeListener(this, controller));
       }
@@ -318,17 +341,22 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
                                    final boolean displayShadowOnFloor) {
     addAncestorListener(new AncestorListener() {        
         public void ancestorAdded(AncestorEvent event) {
-          universe = createUniverse(displayShadowOnFloor, true, false);
+          if (offscreenUniverse != null) {
+            throw new IllegalStateException("Can't listen to home changes offscreen and onscreen at the same time");
+          }
+          onscreenUniverse = createUniverse(displayShadowOnFloor, true, false);
           // Bind universe to canvas3D
-          universe.getViewer().getView().addCanvas3D(canvas3D);
+          onscreenUniverse.getViewer().getView().addCanvas3D(canvas3D);
           canvas3D.setFocusable(false);
           updateNavigationPanelImage();
         }
         
         public void ancestorRemoved(AncestorEvent event) {
-          universe.cleanup();
-          removeHomeListeners();
-          universe = null;
+          if (onscreenUniverse != null) {
+            onscreenUniverse.cleanup();
+            removeHomeListeners();
+            onscreenUniverse = null;
+          }
         }
         
         public void ancestorMoved(AncestorEvent event) {
@@ -336,6 +364,12 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
       });
   }
 
+  @Override
+  public void setVisible(boolean visible) {
+    super.setVisible(visible);
+    this.canvas3D.setVisible(visible);
+  }
+  
   /**
    * Preferences property listener bound to this component with a weak reference to avoid
    * strong link between preferences and this component.  
@@ -355,7 +389,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
         ((UserPreferences)ev.getSource()).removePropertyChangeListener(
             UserPreferences.Property.NAVIGATION_PANEL_VISIBLE, this);
       } else {
-        homeComponent3D.setNavigationPanelVisible((Boolean)ev.getNewValue());
+        homeComponent3D.setNavigationPanelVisible((Boolean)ev.getNewValue() && homeComponent3D.isVisible());
       }
     }
   }
@@ -366,39 +400,49 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   private JComponent createNavigationPanel(Home home, 
                                            UserPreferences preferences, 
                                            HomeController3D controller) {
-    JPanel navigationPanel = new JPanel(new GridBagLayout()) {
-        @Override
-        protected void paintComponent(Graphics g) {
-          Graphics2D g2D = (Graphics2D)g;
-          g2D.setColor(Color.BLACK);
-          g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
-          g2D.drawOval(3, 3, getWidth() - 4, getHeight() - 4);
+    JPanel navigationPanel = new JPanel(new GridBagLayout());
+    String navigationPanelIconPath = preferences.getLocalizedString(HomeComponent3D.class, "navigationPanel.icon");
+    final ImageIcon nagivationPanelIcon = navigationPanelIconPath.length() > 0
+        ? new ImageIcon(HomeComponent3D.class.getResource(navigationPanelIconPath))
+        : null;
+    navigationPanel.setBorder(new Border() {
+        public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
+          if (nagivationPanelIcon != null) {
+            nagivationPanelIcon.paintIcon(c, g, x, y);
+          } else {
+            // Draw a surrounding oval if no navigation panel icon is defined
+            Graphics2D g2D = (Graphics2D)g;
+            g2D.setColor(Color.BLACK);
+            g2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
+            g2D.drawOval(x + 3, y + 3, width - 6, height - 6);
+          }
         }
-      };   
-      
+  
+        public Insets getBorderInsets(Component c) {
+          return new Insets(2, 2, 2, 2);
+        }
+  
+        public boolean isBorderOpaque() {
+          return false;
+        }
+      });
     navigationPanel.setOpaque(false);
-    navigationPanel.add(new NavigationButton(controller, 0, -(float)Math.PI / 36, 0,
-        new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-previous.png"))),
+    navigationPanel.add(new NavigationButton(0, -(float)Math.PI / 36, 0, "TURN_LEFT", preferences, controller),
         new GridBagConstraints(0, 1, 1, 2, 0, 0, 
-            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 5, 0, 0), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, 12.5f, 0, 0, 
-        new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-up.png"))),
+            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 3, 0, 0), 0, 0));
+    navigationPanel.add(new NavigationButton(12.5f, 0, 0, "GO_FORWARD", preferences, controller),
         new GridBagConstraints(1, 0, 1, 1, 0, 0, 
-            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 0, 0, 0), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, 0, (float)Math.PI / 36, 0, 
-        new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-next.png"))),
+            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(3, 0, 0, 0), 0, 0));
+    navigationPanel.add(new NavigationButton(0, (float)Math.PI / 36, 0, "TURN_RIGHT", preferences, controller),
         new GridBagConstraints(2, 1, 1, 2, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 2), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, -12.5f, 0, 0, 
-        new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-down.png"))),
+    navigationPanel.add(new NavigationButton(-12.5f, 0, 0, "GO_BACKWARD", preferences, controller),
         new GridBagConstraints(1, 3, 1, 1, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 2, 0), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, 0, 0, -(float)Math.PI / 100, 
-        new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-up-small.png"))),
+    navigationPanel.add(new NavigationButton(0, 0, -(float)Math.PI / 100, "TURN_UP", preferences, controller),
         new GridBagConstraints(1, 1, 1, 1, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(1, 1, 1, 1), 0, 0));
-    navigationPanel.add(new NavigationButton(controller, 0, 0, (float)Math.PI / 100, 
-        new ImageIcon(HomeComponent3D.class.getResource("resources/icons/tango/go-down-small.png"))),
+    navigationPanel.add(new NavigationButton(0, 0, (float)Math.PI / 100, "TURN_DOWN", preferences, controller),
         new GridBagConstraints(1, 2, 1, 1, 0, 0, 
             GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 1, 0), 0, 0));
     return navigationPanel;
@@ -410,12 +454,18 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   private static class NavigationButton extends JButton {
     private boolean shiftDown;
 
-    public NavigationButton(final HomeController3D controller, 
-                            final float moveDelta, 
+    public NavigationButton(final float moveDelta, 
                             final float yawDelta, 
                             final float pitchDelta, 
-                            Icon icon) {
-      super(icon);
+                            String actionName, 
+                            UserPreferences preferences,
+                            final HomeController3D controller) {
+      super(new ResourceAction(preferences, HomeComponent3D.class, actionName, true) {
+          @Override
+          public void actionPerformed(ActionEvent ev) {
+            // Manage auto repeat button with mouse listener
+          }
+        });
       // Create a darker press icon
       setPressedIcon(new ImageIcon(createImage(new FilteredImageSource(
           ((ImageIcon)getIcon()).getImage().getSource(),
@@ -632,7 +682,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
       int printedImageSize = (int)(printSize / 72 * 150);
       if (this.printedImage == null 
           || this.printedImage.getWidth() != printedImageSize) {
-        try {
+        try {          
           this.printedImage = getOffScreenImage(printedImageSize, printedImageSize);
         } catch (IllegalRenderingStateException ex) {
           // If off screen canvas failed, consider that 3D view page doesn't exist
@@ -661,8 +711,17 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
    * call {@link #endOffscreenImagesCreation() endOffscreenImagesCreation} method to free resources.
    */
   void startOffscreenImagesCreation() {
-    if (this.universe == null) {
-      this.universe = createUniverse(this.displayShadowOnFloor, true, true);
+    if (this.offscreenUniverse == null) {
+      if (this.onscreenUniverse != null) {
+        throw new IllegalStateException("Can't listen to home changes offscreen and onscreen at the same time");
+      }
+      this.offscreenUniverse = createUniverse(this.displayShadowOnFloor, true, true);
+      // Replace textures by clones because Java 3D doesn't accept all the time 
+      // to share textures between offscreen and onscreen environments 
+      Map<Texture, Texture> replacedTextures = new HashMap<Texture, Texture>();
+      for (Enumeration it = this.offscreenUniverse.getLocale().getAllBranchGraphs(); it.hasMoreElements(); ) {
+        cloneTexture((Node)it.nextElement(), replacedTextures);
+      }      
     }
   }
   
@@ -674,11 +733,17 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
     SimpleUniverse offScreenImageUniverse = null;
     try {
       View view;
-      if (this.universe == null) {
+      if (this.offscreenUniverse == null) {
         offScreenImageUniverse = createUniverse(this.displayShadowOnFloor, false, true);
         view = offScreenImageUniverse.getViewer().getView();
+        // Replace textures by clones because Java 3D doesn't accept all the time 
+        // to share textures between offscreen and onscreen environments 
+        Map<Texture, Texture> replacedTextures = new HashMap<Texture, Texture>();
+        for (Enumeration it = offScreenImageUniverse.getLocale().getAllBranchGraphs(); it.hasMoreElements(); ) {
+          cloneTexture((Node)it.nextElement(), replacedTextures);
+        }      
       } else {
-        view = this.universe.getViewer().getView();
+        view = this.offscreenUniverse.getViewer().getView();
       }
       
       // Empty temporarily selection to create the off screen image
@@ -694,14 +759,42 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
     }
   }
   
+  /**
+   * Replace the textures set on node shapes by clones. 
+   */
+  private void cloneTexture(Node node, Map<Texture, Texture> replacedTextures) {
+    if (node instanceof Group) {
+      // Enumerate children
+      Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
+      while (enumeration.hasMoreElements()) {
+        cloneTexture((Node)enumeration.nextElement(), replacedTextures);
+      }
+    } else if (node instanceof Link) {
+      cloneTexture(((Link)node).getSharedGroup(), replacedTextures);
+    } else if (node instanceof Shape3D) {
+      Appearance appearance = ((Shape3D)node).getAppearance();
+      if (appearance != null) {
+        Texture texture = appearance.getTexture();
+        if (texture != null) {
+          Texture replacedTexture = replacedTextures.get(texture);
+          if (replacedTexture == null) {
+            replacedTexture = (Texture)texture.cloneNodeComponent(false);
+            replacedTextures.put(texture, replacedTexture);
+          }
+          appearance.setTexture(replacedTexture);
+        }
+      }
+    } 
+  }
+
   /**
    * Frees unnecessary resources after the creation of a sequence of multiple offscreen images.
    */
   void endOffscreenImagesCreation() {
-    if (this.universe != null) {
-      this.universe.cleanup();
+    if (this.offscreenUniverse != null) {
+      this.offscreenUniverse.cleanup();
       removeHomeListeners();
-      this.universe = null;
+      this.offscreenUniverse = null;
     }
   }
   
@@ -1406,17 +1499,18 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
     Group homeRoot = createHomeRoot();
     // Add walls, pieces and rooms already available 
     for (Room room : this.home.getRooms()) {
-      addObject(homeRoot, room, waitForLoading);
+      addObject(homeRoot, room, listenToHomeUpdates, waitForLoading);
     }    
     for (Wall wall : this.home.getWalls()) {
-      addObject(homeRoot, wall, waitForLoading);
+      addObject(homeRoot, wall, listenToHomeUpdates, waitForLoading);
     }
+    Map<HomePieceOfFurniture, Node> pieces3D = new HashMap<HomePieceOfFurniture, Node>();
     for (HomePieceOfFurniture piece : this.home.getFurniture()) {
-      addObject(homeRoot, piece, waitForLoading);
+      pieces3D.put(piece, addObject(homeRoot, piece, listenToHomeUpdates, waitForLoading));
     }
     
     if (displayShadowOnFloor) {
-      addShadowOnFloor(homeRoot);
+      addShadowOnFloor(homeRoot, pieces3D);
     }
     
     if (listenToHomeUpdates) {
@@ -1463,7 +1557,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
           Wall wall = ev.getItem();
           switch (ev.getType()) {
             case ADD :
-              addObject(group, wall, false);
+              addObject(group, wall, true, false);
               wall.addPropertyChangeListener(wallChangeListener);
               break;
             case DELETE :
@@ -1514,7 +1608,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
           HomePieceOfFurniture piece = (HomePieceOfFurniture)ev.getItem();
           switch (ev.getType()) {
             case ADD :
-              addObject(group, piece, false);
+              addObject(group, piece, true, false);
               piece.addPropertyChangeListener(furnitureChangeListener);
               break;
             case DELETE :
@@ -1614,7 +1708,7 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
             case ADD :
               // Add room to its group at the index indicated by the event 
               // to ensure the 3D rooms are drawn in the same order as in the plan  
-              addObject(group, room, ev.getIndex(), false);
+              addObject(group, room, ev.getIndex(), true, false);
               room.addPropertyChangeListener(roomChangeListener);
               break;
             case DELETE :
@@ -1654,40 +1748,38 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   /**
    * Adds to <code>group</code> a branch matching <code>homeObject</code>.
    */
-  private void addObject(Group group, Selectable homeObject, boolean waitForLoading) {
-    addObject(group, homeObject, -1, waitForLoading);
+  private Node addObject(Group group, Selectable homeObject, boolean listenToHomeUpdates, boolean waitForLoading) {
+    return addObject(group, homeObject, -1, listenToHomeUpdates, waitForLoading);
   }
 
   /**
    * Adds to <code>group</code> a branch matching <code>homeObject</code> at a given <code>index</code>.
    * If <code>index</code> is equal to -1, <code>homeObject</code> will be added at the end of the group.
    */
-  private void addObject(Group group, Selectable homeObject, int index, boolean waitForLoading) {
+  private Node addObject(Group group, Selectable homeObject, int index, 
+                         boolean listenToHomeUpdates, boolean waitForLoading) {
     Object3DBranch object3D = createObject3D(homeObject, waitForLoading);
-    this.homeObjects.put(homeObject, object3D);
+    if (listenToHomeUpdates) {
+      this.homeObjects.put(homeObject, object3D);
+    }
     if (index == -1) {
       group.addChild(object3D);
     } else {
       group.insertChild(object3D, index);
     }
     clearPrintedImageCache();
+    return object3D;
   }
 
   /**
    * Returns the 3D object matching the given home object. If <code>waitForLoading</code> 
    * is <code>true</code> the resources used by the returned 3D object should be ready to be displayed.
+   * @deprecated Subclasses which used to override this method must be updated to create an instance of  
+   *    a {@link Object3DFactory factory} and give it as parameter to the constructor of this class.
    */
-  protected Object3DBranch createObject3D(Selectable homeObject,
-                                          boolean waitForLoading) {
-    if (homeObject instanceof HomePieceOfFurniture) {
-      return new HomePieceOfFurniture3D((HomePieceOfFurniture)homeObject, this.home, true, waitForLoading);
-    } else if (homeObject instanceof Wall) {
-      return new Wall3D((Wall)homeObject, this.home, true, waitForLoading);
-    } else if (homeObject instanceof Room) {
-      return new Room3D((Room)homeObject, this.home, false, false, waitForLoading);
-    } else {
-      throw new IllegalArgumentException("Can't create 3D object for an item of class " + homeObject.getClass());
-    }
+  private Object3DBranch createObject3D(Selectable homeObject,
+                                        boolean waitForLoading) {
+    return (Object3DBranch)this.object3dFactory.createObject3D(this.home, homeObject, waitForLoading);
   }
 
   /**
@@ -1743,12 +1835,12 @@ public class HomeComponent3D extends JComponent implements com.eteks.sweethome3d
   /**
    * Adds to <code>homeRoot</code> a shape matching the shadow of furniture at ground level.
    */
-  private void addShadowOnFloor(Group homeRoot) {
+  private void addShadowOnFloor(Group homeRoot, Map<HomePieceOfFurniture, Node> pieces3D) {
     Area areaOnFloor = new Area();
     // Compute union of the areas of pieces at ground level that are not lights, doors or windows
-    for (Map.Entry<Selectable, Object3DBranch> object3DEntry : this.homeObjects.entrySet()) {
+    for (Map.Entry<HomePieceOfFurniture, Node> object3DEntry : pieces3D.entrySet()) {
       if (object3DEntry.getKey() instanceof HomePieceOfFurniture) {
-        HomePieceOfFurniture piece = (HomePieceOfFurniture)object3DEntry.getKey();
+        HomePieceOfFurniture piece = object3DEntry.getKey();
         // This operation can be lengthy, so give up if thread is interrupted 
         if (Thread.currentThread().isInterrupted()) {
           return;
