@@ -22,9 +22,11 @@ package com.eteks.sweethome3d.viewcontroller;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.lang.ref.WeakReference;
+import java.text.DateFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.Date;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
@@ -41,22 +43,26 @@ import javax.swing.undo.UndoManager;
 import javax.swing.undo.UndoableEdit;
 import javax.swing.undo.UndoableEditSupport;
 
+import com.eteks.sweethome3d.model.AspectRatio;
 import com.eteks.sweethome3d.model.BackgroundImage;
+import com.eteks.sweethome3d.model.Camera;
 import com.eteks.sweethome3d.model.CatalogDoorOrWindow;
 import com.eteks.sweethome3d.model.CatalogLight;
 import com.eteks.sweethome3d.model.CatalogPieceOfFurniture;
 import com.eteks.sweethome3d.model.CatalogTexture;
 import com.eteks.sweethome3d.model.CollectionEvent;
 import com.eteks.sweethome3d.model.CollectionListener;
+import com.eteks.sweethome3d.model.Compass;
 import com.eteks.sweethome3d.model.DimensionLine;
 import com.eteks.sweethome3d.model.DoorOrWindow;
 import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeApplication;
 import com.eteks.sweethome3d.model.HomeDoorOrWindow;
+import com.eteks.sweethome3d.model.HomeEnvironment;
+import com.eteks.sweethome3d.model.HomeFurnitureGroup;
 import com.eteks.sweethome3d.model.HomeLight;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
-import com.eteks.sweethome3d.model.HomeFurnitureGroup;
 import com.eteks.sweethome3d.model.HomeRecorder;
 import com.eteks.sweethome3d.model.InterruptedRecorderException;
 import com.eteks.sweethome3d.model.Label;
@@ -93,6 +99,7 @@ public class HomeController implements Controller {
   private HomeController3D            homeController3D;
   private static HelpController       helpController;  // Only one help controller
   private int                         saveUndoLevel;
+  private boolean                     notUndoableModifications;
   private View                        focusedView;
 
   /**
@@ -265,6 +272,7 @@ public class HomeController implements Controller {
     homeView.setEnabled(HomeView.ActionType.CREATE_LABELS, true);
     homeView.setEnabled(HomeView.ActionType.LOCK_BASE_PLAN, true);
     homeView.setEnabled(HomeView.ActionType.UNLOCK_BASE_PLAN, true);
+    homeView.setEnabled(HomeView.ActionType.MODIFY_COMPASS, true);
     homeView.setEnabled(HomeView.ActionType.IMPORT_BACKGROUND_IMAGE, true);
     BackgroundImage backgroundImage = this.home.getBackgroundImage();
     boolean homeHasBackgroundImage = backgroundImage != null;
@@ -279,6 +287,7 @@ public class HomeController implements Controller {
     homeView.setEnabled(HomeView.ActionType.EXPORT_TO_SVG, true); 
     homeView.setEnabled(HomeView.ActionType.VIEW_FROM_TOP, true);
     homeView.setEnabled(HomeView.ActionType.VIEW_FROM_OBSERVER, true);
+    homeView.setEnabled(HomeView.ActionType.STORE_POINT_OF_VIEW, true);
     homeView.setEnabled(HomeView.ActionType.DETACH_3D_VIEW, true);
     homeView.setEnabled(HomeView.ActionType.ATTACH_3D_VIEW, true);
     homeView.setEnabled(HomeView.ActionType.VIEW_FROM_OBSERVER, true);
@@ -396,6 +405,7 @@ public class HomeController implements Controller {
       
     addCatalogSelectionListener();
     addHomeBackgroundImageListener();
+    addNotUndoableModificationListeners();
     addHomeSelectionListener();
     addFurnitureSortListener();
     addUndoSupportListener();
@@ -603,6 +613,49 @@ public class HomeController implements Controller {
     }
   }
 
+  /**
+   * Adds listeners to track property changes that are not undoable.
+   */
+  private void addNotUndoableModificationListeners() {
+    if (this.home != null) {
+      final PropertyChangeListener notUndoableModificationListener = new PropertyChangeListener() {
+          public void propertyChange(PropertyChangeEvent ev) {
+            notUndoableModifications = true;
+            home.setModified(true);
+          }
+        };
+      this.home.addPropertyChangeListener(Home.Property.STORED_CAMERAS, notUndoableModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.VIDEO_WIDTH, notUndoableModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.VIDEO_ASPECT_RATIO, notUndoableModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.VIDEO_FRAME_RATE, notUndoableModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.VIDEO_QUALITY, notUndoableModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.VIDEO_CAMERA_PATH, notUndoableModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.CEILING_LIGHT_COLOR, notUndoableModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.PHOTO_QUALITY, notUndoableModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.PHOTO_ASPECT_RATIO, notUndoableModificationListener);
+      PropertyChangeListener photoSizeModificationListener = new PropertyChangeListener() {
+          public void propertyChange(PropertyChangeEvent ev) {
+            if (home.getEnvironment().getPhotoAspectRatio() != AspectRatio.VIEW_3D_RATIO) {
+              // Ignore photo size modification with 3D view aspect ratio since it can change for various reasons
+              notUndoableModificationListener.propertyChange(ev);
+            }
+          }
+        };
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.PHOTO_WIDTH, photoSizeModificationListener);
+      this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.PHOTO_HEIGHT, photoSizeModificationListener);
+      PropertyChangeListener timeOrLensModificationListener = new PropertyChangeListener() {
+          public void propertyChange(PropertyChangeEvent ev) {
+            if (ev.getPropertyName().equals(Camera.Property.TIME.name())
+                || ev.getPropertyName().equals(Camera.Property.LENS.name())) {
+              notUndoableModificationListener.propertyChange(ev);
+            }
+          }
+        };
+      this.home.getObserverCamera().addPropertyChangeListener(timeOrLensModificationListener);
+      this.home.getTopCamera().addPropertyChangeListener(timeOrLensModificationListener);
+    }
+  }
+  
   /**
    * Enables or disables action bound to selection. 
    * This method will be called when selection in plan or in catalog changes and when 
@@ -631,6 +684,7 @@ public class HomeController implements Controller {
     boolean homeSelectionContainsOneWall = false;
     boolean homeSelectionContainsOneLabel = false;
     boolean homeSelectionContainsItemsWithText = false;
+    boolean homeSelectionContainsCompass = false;
     FurnitureController furnitureController = getFurnitureController();
     if (!modificationState) {
       for (Selectable item : selectedItems) {
@@ -670,11 +724,12 @@ public class HomeController implements Controller {
       boolean homeSelectionContainsDimensionLines = !Home.getDimensionLinesSubList(selectedItems).isEmpty();
       final List<Label> selectedLabels = Home.getLabelsSubList(selectedItems);
       boolean homeSelectionContainsLabels = !selectedLabels.isEmpty();
+      homeSelectionContainsCompass = selectedItems.contains(this.home.getCompass());
       homeSelectionContainsOneLabel = selectedLabels.size() == 1;
       homeSelectionContainsOneCopiableItemOrMore = 
           homeSelectionContainsFurniture || homeSelectionContainsWalls 
           || homeSelectionContainsRooms || homeSelectionContainsDimensionLines
-          || homeSelectionContainsLabels; 
+          || homeSelectionContainsLabels || homeSelectionContainsCompass; 
       homeSelectionContainsItemsWithText = 
           homeSelectionContainsFurniture || homeSelectionContainsRooms 
           || homeSelectionContainsDimensionLines || homeSelectionContainsLabels;
@@ -783,8 +838,10 @@ public class HomeController implements Controller {
           && this.home.getFurniture().size() > 0);
     } else if (this.focusedView == getPlanController().getView()
                || this.focusedView == getHomeController3D().getView()) {
+      boolean homeContainsOneSelectableItemOrMore = !this.home.isEmpty()
+          || this.home.getCompass().isVisible();
       view.setEnabled(HomeView.ActionType.SELECT_ALL,
-          !modificationState && !this.home.isEmpty());
+          !modificationState && homeContainsOneSelectableItemOrMore);
     } else {
       view.setEnabled(HomeView.ActionType.SELECT_ALL, false);
     }
@@ -838,6 +895,13 @@ public class HomeController implements Controller {
     this.home.addRoomsListener((CollectionListener<Room>)homeItemsListener);
     this.home.addDimensionLinesListener((CollectionListener<DimensionLine>)homeItemsListener);
     this.home.addLabelsListener((CollectionListener<Label>)homeItemsListener);
+    this.home.getCompass().addPropertyChangeListener(new PropertyChangeListener() {
+        public void propertyChange(PropertyChangeEvent ev) {
+          if (Compass.Property.VISIBLE.equals(ev.getPropertyName())) {
+            enableSelectAllAction();
+          }
+        }
+      });
   }
 
   /**
@@ -1044,7 +1108,7 @@ public class HomeController implements Controller {
       view.setUndoRedoName(null, this.undoManager.getRedoPresentationName());
     }
     this.saveUndoLevel--;
-    this.home.setModified(this.saveUndoLevel != 0);
+    this.home.setModified(this.saveUndoLevel != 0 || notUndoableModifications);
   }
   
   /**
@@ -1063,7 +1127,7 @@ public class HomeController implements Controller {
       view.setUndoRedoName(this.undoManager.getUndoPresentationName(), null);
     }
     this.saveUndoLevel++;
-    this.home.setModified(this.saveUndoLevel != 0);
+    this.home.setModified(this.saveUndoLevel != 0 || notUndoableModifications);
   }
 
   /**
@@ -1136,7 +1200,9 @@ public class HomeController implements Controller {
   private void addItems(final List<? extends Selectable> items, 
                         float dx, float dy, final boolean isDropInPlanView, 
                         final String presentationNameKey) {
-    if (!items.isEmpty()) {
+    if (items.size() > 1
+        || (items.size() == 1
+            && !(items.get(0) instanceof Compass))) {
       // Always use selection mode after a drop or a paste operation
       getPlanController().setMode(PlanController.Mode.SELECTION);
       // Start a compound edit that adds walls, furniture, rooms, dimension lines and labels to home
@@ -1386,7 +1452,7 @@ public class HomeController implements Controller {
         try {
           if (this.application.getHomeRecorder().exists(homeName)) {
             recentHomes.add(homeName);
-            if (recentHomes.size() == 4) {
+            if (recentHomes.size() == this.preferences.getRecentHomesMaxCount()) {
               break;
             }
           }
@@ -1443,6 +1509,7 @@ public class HomeController implements Controller {
     // Create a task that deletes home and run postCloseTask
     Runnable closeTask = new Runnable() {
         public void run() {
+          home.setRecovered(false);
           application.deleteHome(home);
           if (postCloseTask != null) {
             postCloseTask.run();
@@ -1450,7 +1517,7 @@ public class HomeController implements Controller {
         }
       };
       
-    if (this.home.isModified()) {
+    if (this.home.isModified()  || this.home.isRecovered()) {
       switch (getView().confirmSave(this.home.getName())) {
         case SAVE   : save(HomeRecorder.Type.DEFAULT, closeTask); // Falls through
         case CANCEL : return;
@@ -1558,7 +1625,9 @@ public class HomeController implements Controller {
         public void run() {
           home.setName(homeName);
           saveUndoLevel = 0;
+          notUndoableModifications = false;
           home.setModified(false);
+          home.setRecovered(false);
           // Update recent homes list
           List<String> recentHomes = new ArrayList<String>(preferences.getRecentHomes());
           int homeNameIndex = recentHomes.indexOf(homeName);
@@ -1744,7 +1813,7 @@ public class HomeController implements Controller {
    */
   public void exit() {
     for (Home home : this.application.getHomes()) {
-      if (home.isModified()) {
+      if (home.isModified() || home.isRecovered()) {
         if (getView().confirmExit()) {
           break;
         } else {
@@ -1754,6 +1823,7 @@ public class HomeController implements Controller {
     }
     // Remove all homes from application
     for (Home home : this.application.getHomes()) {
+      home.setRecovered(false);
       this.application.deleteHome(home);
     }
     // Let application decide what to do when there's no more home
@@ -1913,12 +1983,24 @@ public class HomeController implements Controller {
     planController.setScale(newScale);
   }
 
+  /**
+   * Prompts a name for the current camera and stores it in home.
+   */
+  public void storeCamera() {
+    String now = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM).format(new Date());
+    String name = getView().showStoreCameraDialog(now);
+    if (name != null) {
+      getHomeController3D().storeCamera(name);
+    }
+  }
+
   /**
    * Detaches the given <code>view</code> from home view.
    */
   public void detachView(View view) {
     if (view != null) {
       getView().detachView(view);
+      this.notUndoableModifications = true;
     }
   }
       		
@@ -1928,6 +2010,7 @@ public class HomeController implements Controller {
   public void attachView(View view) {
     if (view != null) {
       getView().attachView(view);
+      this.notUndoableModifications = true;
     }
   }
                 
