@@ -24,6 +24,8 @@ import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
+import java.util.Iterator;
 import java.util.List;
 
 import javax.swing.undo.UndoableEditSupport;
@@ -111,6 +113,55 @@ public class HomeController3D implements Controller {
     this.home.setCamera(this.home.getObserverCamera());
   }
   
+  /**
+   * Stores a clone of the current camera in home under the given <code>name</code>.
+   */
+  public void storeCamera(String name) {
+    Camera camera = this.home.getCamera().clone();
+    camera.setName(name);
+    List<Camera> homeStoredCameras = this.home.getStoredCameras();
+    ArrayList<Camera> storedCameras = new ArrayList<Camera>(homeStoredCameras.size() + 1);
+    storedCameras.addAll(homeStoredCameras);
+    // Don't keep two cameras with the same name or the same location
+    for (Iterator<Camera> it = storedCameras.iterator(); it.hasNext(); ) {
+      Camera storedCamera = it.next();
+      if (name.equals(storedCamera.getName())
+          || (camera.getX() == storedCamera.getX()
+              && camera.getY() == storedCamera.getY()
+              && camera.getZ() == storedCamera.getZ()
+              && camera.getPitch() == storedCamera.getPitch()
+              && camera.getYaw() == storedCamera.getYaw()
+              && camera.getFieldOfView() == storedCamera.getFieldOfView()
+              && camera.getTime() == storedCamera.getTime()
+              && camera.getLens() == storedCamera.getLens())) {
+        it.remove();
+      }
+    }
+    storedCameras.add(0, camera);
+    // Ensure home stored cameras don't contain more than 10 cameras
+    while (storedCameras.size() > 10) {
+      storedCameras.remove(storedCameras.size() - 1);
+    }
+    this.home.setStoredCameras(storedCameras);
+  }
+  
+  /**
+   * Switches to observer or top camera and move camera to the values as the current camera.
+   */
+  public void goToCamera(Camera camera) {
+    if (camera instanceof ObserverCamera) {
+      viewFromObserver();
+    } else {
+      viewFromTop();
+    }
+    this.cameraState.goToCamera(camera);
+    // Reorder cameras
+    ArrayList<Camera> storedCameras = new ArrayList<Camera>(this.home.getStoredCameras());
+    storedCameras.remove(camera);
+    storedCameras.add(0, camera);
+    this.home.setStoredCameras(storedCameras);
+  }
+  
   /**
    * Controls the edition of 3D attributes. 
    */
@@ -193,6 +244,9 @@ public class HomeController3D implements Controller {
 
     public void rotateCameraPitch(float delta) {
     }
+
+    public void goToCamera(Camera camera) {
+    }
   }
   
   // CameraControllerState subclasses
@@ -201,11 +255,11 @@ public class HomeController3D implements Controller {
    * Top camera controller state. 
    */
   private class TopCameraState extends CameraControllerState {
-    private final float MIN_SIZE = 1000;
+    private final Rectangle2D MIN_BOUNDS = new Rectangle2D.Float(0, 0, 1000, 1000);
     
     private Camera      topCamera;
     private Rectangle2D homeBounds;
-    private float       homeHeight;
+    private float       minDistanceToHomeCenter;
     private PropertyChangeListener objectChangeListener = new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent evt) {
           updateCameraFromHomeBounds();
@@ -242,11 +296,6 @@ public class HomeController3D implements Controller {
         }
       };
 
-    public TopCameraState() {
-      this.homeBounds = getHomeBounds();
-      this.homeHeight = getHomeHeight();
-    }
-      
     @Override
     public void enter() {
       this.topCamera = home.getCamera();
@@ -269,12 +318,15 @@ public class HomeController3D implements Controller {
      * Updates camera location from home bounds.
      */
     private void updateCameraFromHomeBounds() {
-      Rectangle2D newHomeBounds = getHomeBounds();
-      float deltaZ = (float)(Math.max(this.homeBounds.getWidth(), this.homeBounds.getHeight())  
-          - Math.max(newHomeBounds.getWidth(), newHomeBounds.getHeight()));
-      this.homeBounds = newHomeBounds;
-      this.homeHeight = getHomeHeight();
-      moveCamera(deltaZ);
+      if (this.homeBounds == null) {
+        this.homeBounds = getHomeBounds();
+      }
+      float distanceToCenter = (float)Math.sqrt(Math.pow(this.homeBounds.getCenterX() - this.topCamera.getX(), 2) 
+          + Math.pow(this.homeBounds.getCenterY() - this.topCamera.getY(), 2) 
+          + Math.pow(this.topCamera.getZ(), 2));
+      this.homeBounds = getHomeBounds();
+      this.minDistanceToHomeCenter = getMinDistanceToHomeCenter(this.homeBounds);
+      placeCameraAt(distanceToCenter);
     }
 
     /**
@@ -283,9 +335,11 @@ public class HomeController3D implements Controller {
     private Rectangle2D getHomeBounds() {
       // Compute plan bounds to include rooms, walls and furniture
       Rectangle2D homeBounds = null;
-      for (Wall wall : home.getWalls()) {
-        homeBounds = updateHomeBounds(homeBounds, wall.getXStart(), wall.getYStart());
-        homeBounds.add(wall.getXEnd(), wall.getYEnd());
+      Collection<Wall> walls = home.getWalls();
+      for (Wall wall : walls) {
+        for (float [] point : wall.getPoints()) {
+          homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
+        }
       }
       for (HomePieceOfFurniture piece : home.getFurniture()) {
         if (piece.isVisible()) {
@@ -301,18 +355,23 @@ public class HomeController3D implements Controller {
       }
 
       if (homeBounds != null) {
-        // Ensure plan bounds are always minimum 10 meters wide centered in middle of 3D view
-        return new Rectangle2D.Float(
-            (float)(MIN_SIZE < homeBounds.getWidth() 
-                        ? homeBounds.getMinX()
-                        : homeBounds.getCenterX() - MIN_SIZE / 2), 
-            (float)(MIN_SIZE < homeBounds.getHeight() 
-                        ? homeBounds.getMinY()
-                        : homeBounds.getCenterY() - MIN_SIZE / 2), 
-            (float)Math.max(MIN_SIZE, homeBounds.getWidth()), 
-            (float)Math.max(MIN_SIZE, homeBounds.getHeight()));
+        if (walls.isEmpty()) {
+          // If home contains only rooms and furniture don't fix minimum size
+          return homeBounds;
+        } else {
+          // Ensure plan bounds are always minimum 10 meters wide centered in middle of 3D view
+          return new Rectangle2D.Float(
+              (float)(MIN_BOUNDS.getWidth() < homeBounds.getWidth() 
+                          ? homeBounds.getMinX()
+                          : homeBounds.getCenterX() - MIN_BOUNDS.getWidth() / 2), 
+              (float)(MIN_BOUNDS.getHeight() < homeBounds.getHeight() 
+                          ? homeBounds.getMinY()
+                          : homeBounds.getCenterY() - MIN_BOUNDS.getHeight() / 2), 
+              (float)Math.max(MIN_BOUNDS.getWidth(), homeBounds.getWidth()), 
+              (float)Math.max(MIN_BOUNDS.getHeight(), homeBounds.getHeight()));
+        }
       } else {
-        return new Rectangle2D.Float(0, 0, MIN_SIZE, MIN_SIZE);
+        return MIN_BOUNDS;
       }
     }
 
@@ -330,47 +389,61 @@ public class HomeController3D implements Controller {
     }
 
     /**
-     * Returns home height that includes walls and furniture.
+     * Returns the minimum distance of the camera to home center.
      */
-    private float getHomeHeight() {
-      // Compute plan bounds to include walls and furniture
-      float homeHeight = 0;
-      for (Wall wall : home.getWalls()) {
-        Float height = wall.getHeight();
-        if (height != null) {
-          homeHeight = Math.max(homeHeight, height);
+    private float getMinDistanceToHomeCenter(Rectangle2D homeBounds) {
+      float maxHeight = 0;
+      Collection<Wall> walls = home.getWalls();
+      if (walls.isEmpty()) {
+        // If home contains no wall, search the max height of the highest piece
+        for (HomePieceOfFurniture piece : home.getFurniture()) {
+          if (piece.isVisible()) {
+            maxHeight = Math.max(maxHeight, piece.getElevation() + piece.getHeight());
+          }
         }
-        Float heightAtEnd = wall.getHeightAtEnd();
-        if (heightAtEnd != null) {
-          homeHeight = Math.max(homeHeight, heightAtEnd);
-        }
-      }
-      for (HomePieceOfFurniture piece : home.getFurniture()) {
-        if (piece.isVisible()) {
-          homeHeight = Math.max(homeHeight, piece.getElevation() + piece.getHeight());
-        }
-      }
-      if (homeHeight > 0) {
-        return Math.max(10, homeHeight);
       } else {
-        return home.getWallHeight();
+         // Search the max height of the highest wall
+        for (Wall wall : walls) {
+          Float height = wall.getHeight();
+          if (height != null) {
+            maxHeight = Math.max(maxHeight, height);
+          }
+          Float heightAtEnd = wall.getHeightAtEnd();
+          if (heightAtEnd != null) {
+            maxHeight = Math.max(maxHeight, heightAtEnd);
+          }
+        }
       }
+      if (maxHeight > 0) {
+        maxHeight = Math.max(10, maxHeight);
+      } else {
+        maxHeight = home.getWallHeight();        
+      }
+      double halfDiagonal = Math.sqrt(homeBounds.getWidth() * homeBounds.getWidth() + homeBounds.getHeight() * homeBounds.getHeight()) / 2;
+      return (float)Math.sqrt(maxHeight * maxHeight + halfDiagonal * halfDiagonal) * 1.1f;
     }
     
     @Override
     public void moveCamera(float delta) {
       // Use a 5 times bigger delta for top camera move
       delta *= 5;
-      float newZ = this.topCamera.getZ() - (float)Math.sin(this.topCamera.getPitch()) * delta;
-      // Check new elevation is between home height and a half, and 3 times its largest dimension  
-      newZ = Math.max(newZ, this.homeHeight * 1.5f);
-      newZ = Math.min(newZ, (float)(Math.max(this.homeBounds.getWidth(), this.homeBounds.getHeight()) * 3 * Math.sin(this.topCamera.getPitch())));
-      double distanceToCenterAtGroundLevel = newZ / Math.tan(this.topCamera.getPitch());
+      float newDistanceToCenter = (float)Math.sqrt(Math.pow(this.homeBounds.getCenterX() - this.topCamera.getX(), 2) 
+          + Math.pow(this.homeBounds.getCenterY() - this.topCamera.getY(), 2) 
+          + Math.pow(this.topCamera.getZ(), 2)) - delta;
+      placeCameraAt(newDistanceToCenter);
+    }
+
+    public void placeCameraAt(float distanceToCenter) {
+      // Check camera is always outside the sphere centered in home center and with a radius equal to minimum distance   
+      distanceToCenter = Math.max(distanceToCenter, this.minDistanceToHomeCenter);
+      // Check camera isn't too far
+      distanceToCenter = Math.min(distanceToCenter, 5 * this.minDistanceToHomeCenter);
+      double distanceToCenterAtGroundLevel = distanceToCenter * Math.cos(this.topCamera.getPitch());
       this.topCamera.setX((float)this.homeBounds.getCenterX() + (float)(Math.sin(this.topCamera.getYaw()) 
           * distanceToCenterAtGroundLevel));
       this.topCamera.setY((float)this.homeBounds.getCenterY() - (float)(Math.cos(this.topCamera.getYaw()) 
           * distanceToCenterAtGroundLevel));
-      this.topCamera.setZ(newZ);
+      this.topCamera.setZ((float)Math.sin(this.topCamera.getPitch()) * distanceToCenter);
     }
 
     @Override
@@ -386,8 +459,8 @@ public class HomeController3D implements Controller {
     @Override
     public void rotateCameraPitch(float delta) {
       float newPitch = this.topCamera.getPitch() - delta;
-      // Check new pitch is between PI / 2 and PI / 8  
-      newPitch = Math.max(newPitch, (float)Math.PI / 8);
+      // Check new pitch is between PI / 2 and PI / 16  
+      newPitch = Math.max(newPitch, (float)Math.PI / 16);
       newPitch = Math.min(newPitch, (float)Math.PI / 2);
       // Compute new z to keep the same distance to view center
       double cameraToBoundsCenterDistance = Math.sqrt(Math.pow(this.topCamera.getX() - this.homeBounds.getCenterX(), 2)
@@ -404,6 +477,14 @@ public class HomeController3D implements Controller {
       this.topCamera.setZ(newZ);
     }
     
+    @Override
+    public void goToCamera(Camera camera) {
+      this.topCamera.setCamera(camera);
+      this.topCamera.setTime(camera.getTime());
+      this.topCamera.setLens(camera.getLens());
+      updateCameraFromHomeBounds();
+    }
+    
     @Override
     public void exit() {
       this.topCamera = null;
@@ -471,6 +552,13 @@ public class HomeController3D implements Controller {
       home.setSelectedItems(Arrays.asList(new Selectable [] {this.observerCamera}));
     }
     
+    @Override
+    public void goToCamera(Camera camera) {
+      this.observerCamera.setCamera(camera);
+      this.observerCamera.setTime(camera.getTime());
+      this.observerCamera.setLens(camera.getLens());
+    }
+    
     @Override
     public void exit() {
       // Remove observer camera from selection
